import asyncio
import logging
import sys
import time
from aiohttp import ClientSession

from pyGeez.Clients import *
from pyGeez.methods import *
from pyGeez.pyrogram import GeezMethods
from pyGeez.pyrogram import eod, eor
from pyGeez.gp import GenSession
from pyGeez.telethon.geez import *


# Bot Logs setup:
logging.basicConfig(
    format="[%(name)s] - [%(levelname)s] - %(message)s",
    level=logging.INFO,
)
logging.getLogger("pyGeez").setLevel(logging.INFO)
logging.getLogger("fipper").setLevel(logging.ERROR)
logging.getLogger("fipper.client").setLevel(logging.ERROR)
logging.getLogger("fipper.session.auth").setLevel(logging.ERROR)
logging.getLogger("fipper.session.session").setLevel(logging.ERROR)


logs = logging.getLogger(__name__)


__copyright__ = "Copyright (C) 2022-present vckyou <https://github.com/vckyou>"
__license__ = "GNU General Public License v3.0 (GPL-3.0)"
__version__ = "0.1"
geez_ver = "0.1"


adB = GeezDB()

DEVS = [874946835]

StartTime = time.time()


class Pyrograms(GeezMethods, GenSession, Methods):
    pass


class Telethons(GeezMethod, GenSession, Methods):
    pass


suc_msg = (f"""
========================×========================
           Credit Py-Geez {__version__}
========================×========================
"""
)

fail_msg = (f"""
========================×========================
      Commit Yang Bener Bego Biar Gak Error
           Credit Py-Geez {__version__}
========================×========================
"""
)

start_bot = (f"""
========================×========================
         Starting GeezUbot Version {geez_ver}
        Copyright (C) 2022-present vckyou
========================×========================
"""
)

run_as_module = False

if sys.argv[0] == "-m":
    run_as_module = True

    from .decorator import *

    print("\n\n" + __copyright__ + "\n" + __license__)
    print(start_bot)

    update_envs()

    CMD_HELP = {}
    adB = GeezDB()
    aiosession = ClientSession()
    loop = asyncio.get_event_loop()
    HOSTED_ON = where_hosted()
else:
    print(suc_msg)
    print("\n\n" + __copyright__ + "\n" + __license__)
    print(fail_msg)

    adB = GeezDB()
    aiosession = ClientSession()
    loop = asyncio.get_event_loop()
    HOSTED_ON = where_hosted()



from ._wrappers import eod, eor
from .func import Function
from .misc import Misc
from .pastebin import PasteBin, paste, post, s_paste
from .sections import section
from .toolbot import ToolBot
from .tools import Tools


class GeezMethods(
    Function,
    Misc,
    ToolBot,
    Tools,
):
    pass



import logging
from datetime import datetime
from traceback import format_exc
import pytz
from fipper import ContinuePropagation, StopPropagation, filters
from fipper.enums import ChatMemberStatus, ChatType
from fipper.errors.exceptions.bad_request_400 import (
    MessageIdInvalid,
    MessageNotModified,
    MessageEmpty,
    UserNotParticipant
)
from fipper.handlers import MessageHandler

from pyGeez.pyrogram import eor

from . import DEVS
from .config import Var as Variable
from .Clients import *


Var = Variable()


async def is_admin_or_owner(message, user_id) -> bool:
    """Check If A User Is Creator Or Admin Of The Current Group"""
    if message.chat.type in [ChatType.PRIVATE, ChatType.BOT]:
        # You Are Boss Of Pvt Chats.
        return True
    user_s = await message.chat.get_member(int(user_id))
    if user_s.status in (
            ChatMemberStatus.OWNER,
            ChatMemberStatus.ADMINISTRATOR):
        return True
    return False


def Geez(
    cmd: list,
    group: int = 0,
    devs: bool = False,
    pm_only: bool = False,
    group_only: bool = False,
    channel_only: bool = False,
    admin_only: bool = False,
    pass_error: bool = False,
    propagate_to_next_handler: bool = True,
):
    """- Main Decorator To Register Commands. -"""
    if not devs:
        filterm = (
            filters.me
            & filters.command(cmd, Var.HNDLR)
            & ~filters.via_bot
            & ~filters.forwarded
        )
    else:
        filterm = (
            filters.user(DEVS)
            & filters.command(cmd, "")
        )

    def decorator(func):
        async def wrapper(client, message):
            message.client = client
            chat_type = message.chat.type
            if admin_only and not await is_admin_or_owner(
                message, (client.me).id
            ):
                await eor(
                    message, "<code>This Command Only Works, If You Are Admin Of The Chat!</code>"
                )
                return
            if group_only and chat_type != (
                    ChatType.GROUP or ChatType.SUPERGROUP):
                await eor(message, "<code>Are you sure this is a group?</code>")
                return
            if channel_only and chat_type != ChatType.CHANNEL:
                await eor(message, "This Command Only Works In Channel!")
                return
            if pm_only and chat_type != ChatType.PRIVATE:
                await eor(message, "<code>This Cmd Only Works On PM!</code>")
                return
            if pass_error:
                await func(client, message)
            else:
                try:
                    await func(client, message)
                except StopPropagation:
                    raise StopPropagation
                except KeyboardInterrupt:
                    pass
                except MessageNotModified:
                    pass
                except MessageIdInvalid:
                    logging.warning(
                        "Please Don't Delete Commands While it's Processing..."
                    )
                except UserNotParticipant:
                    pass
                except ContinuePropagation:
                    raise ContinuePropagation
                except BaseException:
                    logging.error(
                        f"Exception - {func.__module__} - {func.__name__}"
                    )
                    TZZ = pytz.timezone(Var.TZ)
                    datetime_tz = datetime.now(TZZ)
                    text = "<b>!ERROR - REPORT!</b>\n\n"
                    text += f"\n<b>Dari:</b> <code>{client.me.first_name}</code>"
                    text += f"\n<b>Trace Back : </b> <code>{str(format_exc())}</code>"
                    text += f"\n<b>Plugin-Name :</b> <code>{func.__module__}</code>"
                    text += f"\n<b>Function Name :</b> <code>{func.__name__}</code> \n"
                    text += datetime_tz.strftime(
                        "<b>Date :</b> <code>%Y-%m-%d</code> \n<b>Time :</b> <code>%H:%M:%S</code>"
                    )
                    try:
                        xx = await tgbot.send_message(Var.LOG_CHAT, text)
                        await xx.pin(disable_notification=False)
                    except BaseException:
                        logging.error(text)
        add_handler(filterm, wrapper, cmd)
        return wrapper

    return decorator


def listen(filter_s):
    """Simple Decorator To Handel Custom Filters"""
    def decorator(func):
        async def wrapper(client, message):
            try:
                await func(client, message)
            except StopPropagation:
                raise StopPropagation
            except ContinuePropagation:
                raise ContinuePropagation
            except UserNotParticipant:
                pass
            except MessageEmpty:
                pass
            except BaseException:
                logging.error(
                    f"Exception - {func.__module__} - {func.__name__}")
                TZZ = pytz.timezone(Var.TZ)
                datetime_tz = datetime.now(TZZ)
                text = "<b>!ERROR WHILE HANDLING UPDATES!</b>\n\n"
                text += f"\n<b>Dari:</b> <code>{client.me.first_name}</code>"
                text += f"\n<b>Trace Back : </b> <code>{str(format_exc())}</code>"
                text += f"\n<b>Plugin Name :</b> <code>{func.__module__}</code>"
                text += f"\n<b>Function Name :</b> <code>{func.__name__}</code> \n"
                text += datetime_tz.strftime(
                    "<b>Date :</b> <code>%Y-%m-%d</code> \n<b>Time :</b> <code>%H:%M:%S</code>"
                )
                try:
                    xx = await tgbot.send_message(Var.LOG_CHAT, text)
                    await xx.pin(disable_notification=False)
                except BaseException:
                    logging.error(text)
            message.continue_propagation()
        if GEEZ1:
            GEEZ1.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ2:
            GEEZ2.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ3:
            GEEZ3.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ4:
            GEEZ4.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ5:
            GEEZ5.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ6:
            GEEZ6.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ7:
            GEEZ7.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ8:
            GEEZ8.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ9:
            GEEZ9.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ10:
            GEEZ10.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        '''
        if GEEZ11:
            GEEZ11.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ12:
            GEEZ12.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ13:
            GEEZ13.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ14:
            GEEZ14.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ15:
            GEEZ15.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ16:
            GEEZ16.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ17:
            GEEZ17.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ18:
            GEEZ18.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ19:
            GEEZ19.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ20:
            GEEZ20.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ21:
            GEEZ21.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ22:
            GEEZ22.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ23:
            GEEZ23.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ24:
            GEEZ24.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ25:
            GEEZ25.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ26:
            GEEZ26.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ27:
            GEEZ27.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ28:
            GEEZ28.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ29:
            GEEZ29.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ30:
            GEEZ30.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ31:
            GEEZ31.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ32:
            GEEZ32.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ33:
            GEEZ33.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ34:
            GEEZ34.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ35:
            GEEZ35.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ36:
            GEEZ36.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ37:
            GEEZ37.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ38:
            GEEZ38.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ39:
            GEEZ39.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ40:
            GEEZ40.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ41:
            GEEZ41.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ42:
            GEEZ42.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ43:
            GEEZ43.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ44:
            GEEZ44.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ45:
            GEEZ45.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ46:
            GEEZ46.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ47:
            GEEZ47.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ48:
            GEEZ48.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ49:
            GEEZ49.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ50:
            GEEZ50.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ51:
            GEEZ51.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ52:
            GEEZ52.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ53:
            GEEZ53.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ54:
            GEEZ54.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ55:
            GEEZ55.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ56:
            GEEZ56.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ57:
            GEEZ57.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ58:
            GEEZ58.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ59:
            GEEZ59.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ60:
            GEEZ60.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ61:
            GEEZ61.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ62:
            GEEZ62.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ63:
            GEEZ63.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ64:
            GEEZ64.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ65:
            GEEZ65.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ66:
            GEEZ66.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ67:
            GEEZ67.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ68:
            GEEZ68.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ69:
            GEEZ69.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ70:
            GEEZ70.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ71:
            GEEZ71.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ72:
            GEEZ72.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ73:
            GEEZ73.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ74:
            GEEZ74.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ75:
            GEEZ75.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ76:
            GEEZ76.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ77:
            GEEZ77.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ78:
            GEEZ78.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ79:
            GEEZ79.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ80:
            GEEZ80.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ81:
            GEEZ81.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ82:
            GEEZ82.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ83:
            GEEZ83.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ84:
            GEEZ84.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ85:
            GEEZ85.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ86:
            GEEZ86.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ87:
            GEEZ87.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ88:
            GEEZ88.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ89:
            GEEZ89.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ90:
            GEEZ90.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ91:
            GEEZ91.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ92:
            GEEZ92.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ93:
            GEEZ93.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ94:
            GEEZ94.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ95:
            GEEZ95.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ96:
            GEEZ96.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ97:
            GEEZ97.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ98:
            GEEZ98.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ99:
            GEEZ99.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        if GEEZ100:
            GEEZ100.add_handler(
                MessageHandler(
                    wrapper,
                    filters=filter_s),
                group=0)
        '''
        return wrapper

    return decorator


def add_handler(filter_s, func_, cmd):
    if GEEZ1:
        GEEZ1.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ2:
        GEEZ2.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ3:
        GEEZ3.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ4:
        GEEZ4.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ5:
        GEEZ5.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ6:
        GEEZ6.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ7:
        GEEZ7.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ8:
        GEEZ8.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ9:
        GEEZ9.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ10:
        GEEZ10.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    '''
    if GEEZ11:
        GEEZ11.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ12:
        GEEZ12.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ13:
        GEEZ13.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ14:
        GEEZ14.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ15:
        GEEZ15.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ16:
        GEEZ16.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ17:
        GEEZ17.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ18:
        GEEZ18.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ19:
        GEEZ19.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ20:
        GEEZ20.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ21:
        GEEZ21.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ22:
        GEEZ22.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ23:
        GEEZ23.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ24:
        GEEZ24.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ25:
        GEEZ25.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ26:
        GEEZ26.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ27:
        GEEZ27.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ28:
        GEEZ28.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ29:
        GEEZ29.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ30:
        GEEZ30.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ31:
        GEEZ31.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ32:
        GEEZ32.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ33:
        GEEZ33.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ34:
        GEEZ34.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ35:
        GEEZ35.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ36:
        GEEZ36.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ37:
        GEEZ37.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ38:
        GEEZ38.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ39:
        GEEZ39.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ40:
        GEEZ40.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ41:
        GEEZ41.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ42:
        GEEZ42.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ43:
        GEEZ43.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ44:
        GEEZ44.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ45:
        GEEZ45.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ46:
        GEEZ46.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ47:
        GEEZ47.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ48:
        GEEZ48.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ49:
        GEEZ49.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ50:
        GEEZ50.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ51:
        GEEZ51.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ52:
        GEEZ52.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ53:
        GEEZ53.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ54:
        GEEZ54.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ55:
        GEEZ55.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ56:
        GEEZ56.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ57:
        GEEZ57.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ58:
        GEEZ58.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ59:
        GEEZ59.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ60:
        GEEZ60.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ61:
        GEEZ61.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ62:
        GEEZ62.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ63:
        GEEZ63.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ64:
        GEEZ64.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ65:
        GEEZ65.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ66:
        GEEZ66.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ67:
        GEEZ67.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ68:
        GEEZ68.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ69:
        GEEZ69.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ70:
        GEEZ70.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ71:
        GEEZ71.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ72:
        GEEZ72.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ73:
        GEEZ73.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ74:
        GEEZ74.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ75:
        GEEZ75.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ76:
        GEEZ76.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ77:
        GEEZ77.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ78:
        GEEZ78.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ79:
        GEEZ79.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ80:
        GEEZ80.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ81:
        GEEZ81.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ82:
        GEEZ82.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ83:
        GEEZ83.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ84:
        GEEZ84.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ85:
        GEEZ85.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ86:
        GEEZ86.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ87:
        GEEZ87.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ88:
        GEEZ88.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ89:
        GEEZ89.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ90:
        GEEZ90.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ91:
        GEEZ91.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ92:
        GEEZ92.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ93:
        GEEZ93.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ94:
        GEEZ94.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ95:
        GEEZ95.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ96:
        GEEZ96.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ97:
        GEEZ97.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ98:
        GEEZ98.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ99:
        GEEZ99.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    if GEEZ100:
        GEEZ100.add_handler(MessageHandler(func_, filters=filter_s), group=0)
    '''



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

from fipper.types import (
    InlineKeyboardButton,
    InlineKeyboardMarkup,
    InlineQueryResultArticle,
    InputTextMessageContent,
)

from ..config import Var


class InlineBot(Var):
    async def inline_pmpermit(self, ids):
        pm_results = [
            (
                InlineQueryResultArticle(
                    title='PmPermit GeezProject',
                    reply_markup=InlineKeyboardMarkup(
                        [
                            [
                                InlineKeyboardButton(
                                    text='• Approve •',
                                    callback_data=f'terima_{ids}',
                                ),
                                InlineKeyboardButton(
                                    text='• Disapprove •',
                                    callback_data=f'tolak_{ids}',
                                ),
                            ],
                            [
                                InlineKeyboardButton(
                                    text='• Close •',
                                    callback_data=f'close',
                                ),
                            ]
                        ]
                    ),
                    input_message_content=InputTextMessageContent(self.PERMIT_MSG),
                )
            )
        ]
        
        return pm_results
    
    async def approve_pmpermit(
        self,
        cb,
        user_ids,
        OLD_MSG,
    ):
        from ..dB.pmpermit_db import approve_user, is_approved

        if is_approved(user_ids):
            await cb.answer("Pengguna Ini Sudah Ada Di Database.", show_alert=True)
            return
        approve_user(user_ids)
        await cb.edit_message_text("Pesan Anda Diterima")
        if str(user_ids) in OLD_MSG:
            await OLD_MSG[str(user_ids)].delete()
        
    async def disapprove_pmpermit(
        self,
        cb,
        user_ids,
    ):
        from ..dB.pmpermit_db import disapprove_user, is_approved
        
        if not is_approved(user_ids):
            return await cb.answer("Pengguna Ini Tidak Ada Di Database")
        disapprove_user(user_ids)
        await cb.edit_message_text("Pesan Anda Ditolak Tod")



# Ultroid - UserBot
# Copyright (C) 2021-2022 TeamUltroid
#
# This file is a part of < https://github.com/TeamUltroid/Ultroid/ >
# PLease read the GNU Affero General Public License in
# <https://github.com/TeamUltroid/pyUltroid/blob/main/LICENSE>.

import asyncio
import multiprocessing
import os
from concurrent.futures import ThreadPoolExecutor
from functools import partial, wraps


def update_envs():
    """Update Var. attributes to adB"""
    from .. import adB

    for envs in list(os.environ):
        if envs in ["LOG_CHAT", "BOT_TOKEN"] or envs in adB.keys():
            adB.set_key(envs, os.environ[envs])


def run_async(function):
    @wraps(function)
    async def wrapper(*args, **kwargs):
        return await asyncio.get_event_loop().run_in_executor(
            ThreadPoolExecutor(max_workers=multiprocessing.cpu_count() * 5),
            partial(function, *args, **kwargs),
        )

    return wrapper



import logging
import sys

from pyGeez.config import Var as Variable

from ..methods._database import GeezDB
from ..methods.helpers import Helpers
from ..methods.hosting import where_hosted

from .client import *


adB = GeezDB()
logs = logging.getLogger(__name__)
HOSTED_ON = where_hosted()
Var = Variable()
xx = Helpers()


async def geez_client(client):
    try:
        await client.join_chat("GeezSupport")
        await client.join_chat("GeezProject")
        await client.join_chat("uc0de")
    except Exception:
        pass


clients = []
client_id = []


async def StartPyrogram():
    try:
        bot_plugins = xx.import_module(
            "assistant/",
            display_module=False,
            exclude=Var.NO_LOAD,
        )
        logs.info(f"{bot_plugins} Total Plugins Bot")
        plugins = xx.import_module(
            "xnxx/",
            display_module=False,
            exclude=Var.NO_LOAD,
        )
        logs.info(f"{plugins} Total Plugins User")
    except BaseException as e:
        logs.info(e)
        sys.exit()
    if tgbot:
        await tgbot.start()
        me = await tgbot.get_me()
        tgbot.id = me.id
        tgbot.mention = me.mention
        tgbot.username = me.username
        if me.last_name:
            tgbot.name = me.first_name + " " + me.last_name
        else:
            tgbot.name = me.first_name
        logs.info(
            f"TgBot in {tgbot.name} | [ {tgbot.id} ]"
        )
        client_id.append(tgbot.id)
    if GEEZ1:
        try:
            await GEEZ1.start()
            clients.append(1)
            await geez_client(GEEZ1)
            me = await GEEZ1.get_me()
            GEEZ1.id = me.id
            GEEZ1.mention = me.mention
            GEEZ1.username = me.username
            if me.last_name:
                GEEZ1.name = me.first_name + " " + me.last_name
            else:
                GEEZ1.name = me.first_name
            #GEEZ1.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ1 in {GEEZ1.name} | [ {GEEZ1.id} ]"
            )
            client_id.append(GEEZ1.id)
        except Exception as e:
            clients.remove(1)
            client_id.remove(GEEZ1.id)
            logs.info(f"[STRING_1] ERROR: {e}")
    if GEEZ2:
        try:
            await GEEZ2.start()
            clients.append(2)
            await geez_client(GEEZ2)
            me = await GEEZ2.get_me()
            GEEZ2.id = me.id
            GEEZ2.mention = me.mention
            GEEZ2.username = me.username
            if me.last_name:
                GEEZ2.name = me.first_name + " " + me.last_name
            else:
                GEEZ2.name = me.first_name
            #GEEZ2.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ2 in {GEEZ2.name} | [ {GEEZ2.id} ]"
            )
            client_id.append(GEEZ2.id)
        except Exception as e:
            clients.remove(2)
            client_id.remove(GEEZ2.id)
            logs.info(f"[STRING_2] ERROR: {e}")
    if GEEZ3:
        try:
            await GEEZ3.start()
            clients.append(3)
            await geez_client(GEEZ3)
            me = await GEEZ3.get_me()
            GEEZ3.id = me.id
            GEEZ3.mention = me.mention
            GEEZ3.username = me.username
            if me.last_name:
                GEEZ3.name = me.first_name + " " + me.last_name
            else:
                GEEZ3.name = me.first_name
            #GEEZ3.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ3 in {GEEZ3.name} | [ {GEEZ3.id} ]"
            )
            client_id.append(GEEZ3.id)
        except Exception as e:
            clients.remove(3)
            client_id.remove(GEEZ3.id)
            logs.info(f"[STRING_3] ERROR: {e}")
    if GEEZ4:
        try:
            await GEEZ4.start()
            clients.append(4)
            await geez_client(GEEZ4)
            me = await GEEZ4.get_me()
            GEEZ4.id = me.id
            GEEZ4.mention = me.mention
            GEEZ4.username = me.username
            if me.last_name:
                GEEZ4.name = me.first_name + " " + me.last_name
            else:
                GEEZ4.name = me.first_name
            #GEEZ4.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ4 in {GEEZ4.name} | [ {GEEZ4.id} ]"
            )
            client_id.append(GEEZ4.id)
        except Exception as e:
            clients.remove(4)
            client_id.remove(GEEZ4.id)
            logs.info(f"[STRING_4] ERROR: {e}")
    if GEEZ5:
        try:
            await GEEZ5.start()
            clients.append(5)
            await geez_client(GEEZ5)
            me = await GEEZ5.get_me()
            GEEZ5.id = me.id
            GEEZ5.mention = me.mention
            GEEZ5.username = me.username
            if me.last_name:
                GEEZ5.name = me.first_name + " " + me.last_name
            else:
                GEEZ5.name = me.first_name
            #GEEZ5.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ5 in {GEEZ5.name} | [ {GEEZ5.id} ]"
            )
            client_id.append(GEEZ5.id)
        except Exception as e:
            clients.remove(5)
            client_id.remove(GEEZ5.id)
            logs.info(f"[STRING_5] ERROR: {e}")
    if GEEZ6:
        try:
            await GEEZ6.start()
            clients.append(6)
            await geez_client(GEEZ6)
            me = await GEEZ6.get_me()
            GEEZ6.id = me.id
            GEEZ6.mention = me.mention
            GEEZ6.username = me.username
            if me.last_name:
                GEEZ6.name = me.first_name + " " + me.last_name
            else:
                GEEZ6.name = me.first_name
            #GEEZ1.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ6 in {GEEZ6.name} | [ {GEEZ6.id} ]"
            )
            client_id.append(GEEZ6.id)
        except Exception as e:
            clients.remove(6)
            client_id.remove(GEEZ6.id)
            logs.info(f"[STRING_6] ERROR: {e}")
    if GEEZ7:
        try:
            await GEEZ7.start()
            clients.append(7)
            await geez_client(GEEZ7)
            me = await GEEZ7.get_me()
            GEEZ7.id = me.id
            GEEZ7.mention = me.mention
            GEEZ7.username = me.username
            if me.last_name:
                GEEZ7.name = me.first_name + " " + me.last_name
            else:
                GEEZ7.name = me.first_name
            #GEEZ7.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ7 in {GEEZ7.name} | [ {GEEZ7.id} ]"
            )
            client_id.append(GEEZ7.id)
        except Exception as e:
            clients.remove(7)
            client_id.remove(GEEZ7.id)
            logs.info(f"[STRING_7] ERROR: {e}")
    if GEEZ8:
        try:
            await GEEZ8.start()
            clients.append(8)
            await geez_client(GEEZ8)
            me = await GEEZ8.get_me()
            GEEZ8.id = me.id
            GEEZ8.mention = me.mention
            GEEZ8.username = me.username
            if me.last_name:
                GEEZ8.name = me.first_name + " " + me.last_name
            else:
                GEEZ8.name = me.first_name
            #GEEZ8.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ8 in {GEEZ8.name} | [ {GEEZ8.id} ]"
            )
            client_id.append(GEEZ8.id)
        except Exception as e:
            clients.remove(8)
            client_id.remove(GEEZ8.id)
            logs.info(f"[STRING_8] ERROR: {e}")
    if GEEZ9:
        try:
            await GEEZ9.start()
            clients.append(9)
            await geez_client(GEEZ9)
            me = await GEEZ9.get_me()
            GEEZ9.id = me.id
            GEEZ9.mention = me.mention
            GEEZ9.username = me.username
            if me.last_name:
                GEEZ9.name = me.first_name + " " + me.last_name
            else:
                GEEZ9.name = me.first_name
            #GEEZ9.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ9 in {GEEZ9.name} | [ {GEEZ9.id} ]"
            )
            client_id.append(GEEZ9.id)
        except Exception as e:
            clients.remove(9)
            client_id.remove(GEEZ9.id)
            logs.info(f"[STRING_9] ERROR: {e}")
    if GEEZ10:
        try:
            await GEEZ10.start()
            clients.append(10)
            await geez_client(GEEZ10)
            me = await GEEZ10.get_me()
            GEEZ10.id = me.id
            GEEZ10.mention = me.mention
            GEEZ10.username = me.username
            if me.last_name:
                GEEZ10.name = me.first_name + " " + me.last_name
            else:
                GEEZ10.name = me.first_name
            #GEEZ10.has_a_bot = True if tgbot else False
            logs.info(
                f"GEEZ10 in {GEEZ10.name} | [ {GEEZ10.id} ]"
            )
            client_id.append(GEEZ10.id)
        except Exception as e:
            clients.remove(10)
            client_id.remove(GEEZ10.id)
            logs.info(f"[STRING_10] ERROR: {e}")
    logs.info(f"Connecting Database To {adB.name}")
    if adB.ping():
        logs.info(f"Succesfully Connect On {adB.name}")
    logs.info(
        f"Connect On [ {HOSTED_ON} ]\n"
    )



from fipper import Client

from ..config import Var as Variable

Var = Variable()

hndlr = f"{Var.HNDLR[0]} {Var.HNDLR[1]} {Var.HNDLR[2]} {Var.HNDLR[3]} {Var.HNDLR[4]} {Var.HNDLR[5]}"


tgbot = (
    Client(
        name="tgbot",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        bot_token=Var.BOT_TOKEN,
    )
)

# For Publik Repository
GEEZ1 = (
    Client(
        name="GEEZ1",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_1,
        in_memory=True,
    )
    if Var.STRING_1
    else None
)


GEEZ2 = (
    Client(
        name="GEEZ2",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_2,
        in_memory=True,
    )
    if Var.STRING_2
    else None
)
        
GEEZ3 = (
    Client(
        name="GEEZ3",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_3,
        in_memory=True,
    )
    if Var.STRING_3
    else None
)

GEEZ4 = (
    Client(
        name="GEEZ4",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_4,
        in_memory=True,
    )
    if Var.STRING_4
    else None
)

GEEZ5 = (
    Client(
        name="GEEZ5",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_5,
        in_memory=True,
    )
    if Var.STRING_5
    else None
)

GEEZ6 = (
    Client(
        name="GEEZ6",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_6,
        in_memory=True,
    )
    if Var.STRING_6
    else None
)


GEEZ7 = (
    Client(
        name="GEEZ7",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_7,
        in_memory=True,
    )
    if Var.STRING_7
    else None
)
        
GEEZ8 = (
    Client(
        name="GEEZ8",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_8,
        in_memory=True,
    )
    if Var.STRING_8
    else None
)


GEEZ9 = (
    Client(
        name="GEEZ9",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_9,
        in_memory=True,
    )
    if Var.STRING_9
    else None
)
GEEZ10 = (
    Client(
        name="GEEZ10",
        api_id=Var.API_ID,
        api_hash=Var.API_HASH,
        session_string=Var.STRING_10,
        in_memory=True,
    )
    if Var.STRING_10
    else None
)


Bots = [
    bot for bot in [
        GEEZ1, 
        GEEZ2, 
        GEEZ3, 
        GEEZ4, 
        GEEZ5, 
        GEEZ6, 
        GEEZ7, 
        GEEZ8,
        GEEZ9,
        GEEZ10,
    ] if bot
]



import asyncio
import importlib
import os
import ssl
import sys

from fipper import Client
from ..pyrogram import eor, section
from ..Clients import *

from .func import run_async


try:
    import certifi
except ImportError:
    certifi = None


try:
    from git import Repo
    from git.exc import GitCommandError, InvalidGitRepositoryError, NoSuchPathError
except ImportError:
    Repo = None


try:
    import heroku3
except ImportError:
    heroku3 = None


class Helpers(object):
    def import_module(
        self,
        path: str,
        exclude: list = [],
        display_module: bool = True
    ):
    
        listbin = []
        listbin.clear()
    
        if not os.path.exists(path):
            return print(f"No path found: {path}")
    
        modules = []
        modules.clear()
    
        for x in os.listdir(path):
            if x.endswith(".py"):
                if x not in ["__pycache__", "__init__.py"]:
                    modules.append(x.replace(".py", ""))
    
        py_path_raw = ".".join(path.split("/"))
        py_path = py_path_raw[0:len(py_path_raw) - 1]
    
        count = 0
        for x in modules:
            if x not in exclude:
                importlib.import_module(py_path + "." + x)
                count += 1
                listbin.append(x)
    
        if display_module:
            data = sorted(listbin)
            for x in data:
                print(x + " Loaded !")
        return count


    async def get_paste(
        self,
        data: str,
        extension: str = "txt"
    ):
        ssl_context = ssl.create_default_context(cafile=certifi.where())
        json = {"content": data, "extension": extension}
        key = await self.async_searcher(
            url="https://spaceb.in/api/v1/documents/",
            json=json,
            ssl=ssl_context,
            post=True,
            re_json=True,
        )
        try:
            return True, key["payload"]["id"]
        except KeyError:
            if "the length must be between 2 and 400000." in key["error"]:
                return await self.get_paste(data[-400000:], extension=extension)
            return False, key["error"]
        except Exception as e:
            print(e)
            return None, str(e)

    async def install_requirements(self):
        try:
            process = await asyncio.create_subprocess_shell(
                " ".join([sys.executable, "-m", "pip", "install", '-U', "-r", "requirements.txt"]),
                stdout=asyncio.subprocess.PIPE,
                stderr=asyncio.subprocess.PIPE,
            )
            await process.communicate()
            return process.returncode
        except Exception as e:
            return repr(e)
    
    @run_async
    def gen_chlog(self, repo, diff):
        """Generate Changelogs..."""
        from pyGeez import geez_ver
        
        UPSTREAM_REPO_URL = (
            Repo().remotes[0].config_reader.get("url").replace(".git", "")
        )
        ac_br = repo.active_branch.name
        ch_log = tldr_log = ""
        ch = f"<b>GeezProject {geez_ver} updates for <a href={UPSTREAM_REPO_URL}/tree/{ac_br}>[{ac_br}]</a>:</b>"
        ch_tl = f"GeezProject {geez_ver} updates for {ac_br}:"
        d_form = "%d/%m/%y || %H:%M"
        for c in repo.iter_commits(diff):
            ch_log += f"\n\n# <b>{c.count()}</b> 🗓 <b>[{c.committed_datetime.strftime(d_form)}]</b>\n<b><a href={UPSTREAM_REPO_URL.rstrip('/')}/commit/{c}>[{c.summary}]</a></b> 👨‍💻 <code>{c.author}</code>"
            tldr_log += f"\n\n# {c.count()} 🗓 [{c.committed_datetime.strftime(d_form)}]\n[{c.summary}] 👨‍💻 {c.author}"
        if ch_log:
            return str(ch + ch_log), str(ch_tl + tldr_log)
        return ch_log, tldr_log
    
    async def updater(self):
        from .. import logs

        try:
            off_repo = Repo().remotes[0].config_reader.get("url").replace(".git", "")
        except Exception as er:
            logs.exception(er)
            return
        try:
            repo = Repo()
        except NoSuchPathError as error:
            logs.info(f"`directory {error} is not found`")
            Repo().__del__()
            return
        except GitCommandError as error:
            logs.info(f"`Early failure! {error}`")
            Repo().__del__()
            return
        except InvalidGitRepositoryError:
            repo = Repo.init()
            origin = repo.create_remote("upstream", off_repo)
            origin.fetch()
            repo.create_head("main", origin.refs.main)
            repo.heads.main.set_tracking_branch(origin.refs.main)
            repo.heads.main.checkout(True)
        ac_br = repo.active_branch.name
        try:
            repo.create_remote("upstream", off_repo)
        except Exception as er:
            logs.info(er)
        ups_rem = repo.remote("upstream")
        ups_rem.fetch(ac_br)
        changelog, tl_chnglog = await self.gen_chlog(repo, f"HEAD..upstream/{ac_br}")
        return bool(changelog)
    
    async def restart(gp=None):
        from pyGeez import logs
        
        if Var.HEROKU_APP_NAME and Var.HEROKU_API:
            try:
                Heroku = heroku3.from_key(Var.HEROKU_API)
                app = Heroku.apps()[Var.HEROKU_APP_NAME]
                if gp:
                    await gp.edit("`Restarting your app, please wait for a minute!`")
                app.restart()
            except BaseException as er:
                if gp:
                    return await eor(
                        gp,
                        "`HEROKU_API` or `HEROKU_APP_NAME` is wrong! Kindly re-check in config vars.",
                    )
                logs.exception(er)
        else:
            if len(sys.argv) == 1:
                os.execl(sys.executable, sys.executable, "-m", "pyGeez")
            else:
                os.execl(
                    sys.executable,
                    sys.executable,
                    "-m",
                    "pyGeez",
                    sys.argv[1],
                    sys.argv[2],
                    sys.argv[3],
                    sys.argv[4],
                    sys.argv[5],
                    sys.argv[6],
                )
    
    async def get_user_info(self, client: Client, user, already=False):
        if not already:
            user = await client.get_users(user)
        if not user.first_name:
            return ["Deleted account", None]
        user_id = user.id
        username = user.username
        first_name = user.first_name
        mention = user.mention
        dc_id = user.dc_id
        photo_id = user.photo.big_file_id if user.photo else None
        body = {
            "ID": user_id,
            "DC": dc_id,
            "Name": [first_name],
            "Username": [("@" + username) if username else ""],
            "Mention": [mention],
        }
        caption = section("User info", body)
        return [caption, photo_id]


    async def get_chat_info(self, client: Client, chat, already=False):
        if not already:
            chat = await client.get_chat(chat)
        chat_id = chat.id
        username = chat.username
        title = chat.title
        type_ = chat.type
        is_scam = chat.is_scam
        description = chat.description
        members = chat.members_count
        is_restricted = chat.is_restricted
        link = f"[Link](t.me/{username})" if username else "??"
        dc_id = chat.dc_id
        photo_id = chat.photo.big_file_id if chat.photo else None
        body = {
            "ID": chat_id,
            "DC": dc_id,
            "Type": type_,
            "Name": [title],
            "Username": [("@" + username) if username else ""],
            "Mention": [link],
            "Members": members,
            "Scam": is_scam,
            "Restricted": is_restricted,
            "Description": [description],
        }
        caption = section("Chat info", body)
        return [caption, photo_id]



from ._database import GeezDB
from ._misc import _Misc
from .converter import Convert
from .func import update_envs
from .helpers import Helpers
from .hosting import where_hosted
from .Inline import InlineBot
from .queue import Queue


class Methods(
    _Misc,
    Convert,
    InlineBot,
    Helpers,
    Queue,
):
    pass
